def get_count(queryset):
    """Determine an object count in optimized manner. Incompatible with custom values/distinct/group by"""
    try:
        return queryset.values('id').order_by().count()
    except (AttributeError, TypeError):
        return len(queryset)



from urllib.parse import parse_qs
from urllib.parse import urlencode
from urllib.parse import urlsplit
from urllib.parse import urlunsplit

from django.utils.encoding import force_str


def simple_urljoin(*args):
    """
        Joins url parts like 'https://', 'google.com/', '/search/' to https://google.com/search/

        Treats parts ending on double slash as url beginning and ignores all parts before them.
        Other parts are treated as path and joined with single slash.

        Preserves single trailing and leading slash.
    """
    sep = '/'
    protocol_sep = '://'
    res = ''

    for idx, piece in enumerate(args):
        is_first = idx == 0
        is_last = idx == len(args) - 1

        add_leading_slash = add_trailing_slash = False

        piece = force_str(piece)

        if is_first and piece.startswith(sep):
            add_leading_slash = True

        if is_last and piece.endswith(sep):
            add_trailing_slash = True

        if not is_last:
            add_trailing_slash = True

        if piece.endswith(protocol_sep):
            piece = piece.lstrip('/')
            add_trailing_slash = False
        else:
            piece = piece.strip('/')

        if add_leading_slash:
            piece = sep + piece

        if add_trailing_slash:
            piece += sep

        if protocol_sep in piece:
            res = piece
        else:
            res += piece

    return res


def add_query(url: str, **params: str):
    scheme, netloc, path, query_string, fragment = urlsplit(url)
    query_params = parse_qs(query_string)

    query_params.update(params)
    new_query_string = urlencode(query_params, doseq=True)

    return urlunsplit((scheme, netloc, path, new_query_string, fragment))



from typing import List, Type

from django.db import models
from manager_utils import bulk_upsert as original_bulk_upsert

from django_handy.unique import get_unique_objs


def get_bulk_update_fields(cls: Type[models.Model], unique_fields: List[str]) -> List[str]:
    """
        Return list of all model fields that can be updated in bulk,
        excluding unique_fields
    """

    def _should_be_updated(field):
        return (
            field.name not in unique_fields and
            not (field.many_to_many or field.one_to_many)
            and not field.auto_created
        )

    return [
        field.name for field in cls._meta.get_fields()
        if _should_be_updated(field)
    ]


not_provided = object()


def bulk_upsert(
    queryset: models.QuerySet, model_objs: List[models.Model],
    unique_fields: List[str], update_fields: List[str] = not_provided,
    return_upserts: bool = False, return_upserts_distinct: bool = False,
    sync: bool = False, native: bool = False
):
    if update_fields is not_provided:
        update_fields = get_bulk_update_fields(queryset.model, unique_fields)

    # noinspection PyTypeChecker
    return original_bulk_upsert(
        queryset,
        model_objs=model_objs,
        unique_fields=unique_fields,
        update_fields=update_fields,
        return_upserts=return_upserts,
        return_upserts_distinct=return_upserts_distinct,
        sync=sync,
        native=native
    )


def safe_bulk_upsert(
    queryset: models.QuerySet, model_objs: List[models.Model],
    unique_fields: List[str], update_fields: List[str] = not_provided,
    return_upserts: bool = False, return_upserts_distinct: bool = False,
    sync: bool = False, native: bool = False
):
    """
        Removes objs with duplicate unique fields to prevent IntegrityError.
        Uses first unique obj encountered
    """
    # noinspection PyTypeChecker
    return bulk_upsert(
        queryset,
        model_objs=get_unique_objs(model_objs, unique_fields),
        unique_fields=unique_fields,
        update_fields=update_fields,
        return_upserts=return_upserts,
        return_upserts_distinct=return_upserts_distinct,
        sync=sync,
        native=native
    )



import mimetypes
from urllib.parse import quote

from django.http import HttpResponse


def create_attachment_response(filename, content: bytes):
    """
        Creates response to download file with correct headers
         for given content and filename
    """
    response = HttpResponse(content=content)
    safe_filename = quote(filename)
    response['Content-Disposition'] = f'inline; filename="{safe_filename}"'

    mime_type, encoding = mimetypes.guess_type(filename)
    if mime_type is None:
        mime_type = 'application/octet-stream'
    response['Content-Type'] = mime_type

    response['Access-Control-Expose-Headers'] = 'Content-Disposition'

    if encoding is not None:
        response['Content-Encoding'] = encoding

    return response



from PIL import Image
from django.core.validators import BaseValidator
from django.forms import forms
from django.template.defaultfilters import filesizeformat
from django.utils.deconstruct import deconstructible
from django.utils.translation import ugettext as _


class MinValueExclusionValidator(BaseValidator):
    message = _('Ensure this value is greater than %(limit_value)s')
    code = 'min_value_exclusive'

    def compare(self, a, b):
        return a <= b


class MaxValueExclusionValidator(BaseValidator):
    message = _('Ensure this value is less than %(limit_value)s')
    code = 'max_value_exclusive'

    def compare(self, a, b):
        return a >= b


@deconstructible
class FileSizeValidator:
    def __init__(self, max_size):
        self.max_size = max_size

    def __call__(self, value):
        if value.size > self.max_size:
            raise forms.ValidationError(
                _('Maximum allowed size is %(max_size)s') % {'max_size': filesizeformat(self.max_size)}
            )


@deconstructible
class ImageSizeValidator(FileSizeValidator):
    def __init__(self, max_size, max_pixels):
        super().__init__(max_size=max_size)
        self.max_pixels = max_pixels

    def __call__(self, value):
        super().__call__(value)

        try:
            image = value.image
        except AttributeError:
            image = Image.open(value)

        dimensions = image.size
        if dimensions[0] > self.max_pixels:
            raise forms.ValidationError(
                _('Maximum allowed width is %(max_pixels)spx') % {'max_pixels': self.max_pixels}
            )
        elif dimensions[1] > self.max_pixels:
            raise forms.ValidationError(
                _('Maximum allowed height is %(max_pixels)spx') % {'max_pixels': self.max_pixels}
            )



from django.db import models
from django.db.models import BooleanField
from django.db.models import ExpressionWrapper
from django.db.models import Subquery


class BooleanQ(ExpressionWrapper):
    output_field = BooleanField()

    def __init__(self, *args, **kwargs):
        expression = models.Q(*args, **kwargs)
        super().__init__(expression, output_field=None)


class SubquerySum(Subquery):
    template = '(SELECT SUM(%(field_name)s) FROM (%(subquery)s) _sub)'
    output_field = models.IntegerField()

    def __init__(self, queryset, field_name, **kwargs):
        queryset = queryset.order_by()
        if not queryset.query.values_select:
            queryset = queryset.values('field_name')
        super().__init__(queryset, field_name=field_name, **kwargs)


class SubqueryCount(Subquery):
    template = '(SELECT COUNT(*) FROM (%(subquery)s) _sub)'
    output_field = models.IntegerField()

    def __init__(self, queryset, **kwargs):
        queryset = queryset.order_by()
        if not queryset.query.values_select:
            queryset = queryset.values('pk')
        super().__init__(queryset, **kwargs)



from decimal import Decimal


def d_round(value, places=2):
    """Decimal version of round() builtin"""
    assert isinstance(places, int)
    quantize_to = Decimal(10) ** (-places)
    return Decimal(value).quantize(quantize_to)



from django.db import models

from rest_framework import serializers


# noinspection PyAbstractClass
class EmptySerializer(serializers.Serializer):
    pass


# noinspection PyUnresolvedReferences
class SerializerRequestMixin:
    @property
    def request(self):
        return self.context['request']


class SelectObjectsSerializer(serializers.Serializer):
    ids = serializers.ListField(
        child=serializers.IntegerField(min_value=0),
        min_length=1,
    )

    @property
    def selected_objects(self) -> models.QuerySet:
        objects = self.context['view'].get_queryset()
        ids = self.validated_data.get('ids', [])
        objects = objects.filter(id__in=ids)
        return objects



from itertools import chain

from django.contrib.admin.utils import flatten_fieldsets
from django.db.models.fields import Field
from django.urls import reverse
from django.utils.html import format_html
from django.utils.inspect import get_func_args

from django_handy.attrs import get_attribute, has_attribute


class ChangeUrl:
    def __init__(self, field, description=None):
        self.field = field
        self.short_description = description or field

    def __call__(self, obj):
        try:
            obj = get_attribute(obj, self.field)
        except AttributeError:
            return '-'

        if not obj:
            return '-'

        url = object_admin_rel_url(obj)
        return format_html('<a href="{}">{}</a>', url, obj)


def object_admin_rel_url(obj):
    return reverse('admin:%s_%s_change' % (obj._meta.app_label, obj._meta.model_name), args=(obj.pk,))


def foreign_field(field_name, short_description=None, admin_order_field=None, boolean=False):
    assert '__' not in field_name, 'use . as delimiter for traversing relations'

    def accessor(obj):
        if not has_attribute(obj, field_name):
            return None
        attribute = get_attribute(obj, field_name)
        if callable(attribute):
            return attribute()
        return attribute

    # Set attribute so we can get is outside
    accessor.attribute = field_name
    short_description = short_description or field_name.replace('.', ' ').replace('_', ' ').capitalize()
    accessor.short_description = short_description
    accessor.admin_order_field = admin_order_field
    accessor.boolean = boolean
    return accessor


class ReadOnlyFieldsAdminMixin:
    editable_fields = []
    exclude = []

    def get_readonly_fields(self, request, obj=None):
        args = get_func_args(self.has_add_permission)
        if 'obj' in args:
            has_add_permission = self.has_add_permission(request, obj)
        else:
            has_add_permission = self.has_add_permission(request)

        if obj is None and has_add_permission:
            return super().get_readonly_fields(request, obj)

        if self.fields or self.fieldsets:
            fields = flatten_fieldsets(self.get_fieldsets(request, obj))
        else:
            opts = self.model._meta
            sortable_private_fields = [f for f in opts.private_fields if isinstance(f, Field)]

            fields = [
                field.name for field in
                sorted(chain(opts.concrete_fields, sortable_private_fields, opts.many_to_many))
                if field.editable and not field.auto_created
            ]

        exclude = self.get_exclude(request, obj)
        editable_fields = self.get_editable_fields(request, obj)
        return [
            field for field in fields
            if field not in editable_fields and field not in exclude
        ]

    def get_editable_fields(self, request, obj=None):
        return self.editable_fields


class NoAddDeleteAdminMixin:
    def has_add_permission(self, request, obj=None):
        return False

    def has_delete_permission(self, request, obj=None):
        return False


class ReadOnlyAdminMixin(ReadOnlyFieldsAdminMixin, NoAddDeleteAdminMixin):
    def has_change_permission(self, request, obj=None):
        if hasattr(self, 'has_view_permission'):  # Django >= 2.1
            return False
        return super().has_change_permission(request, obj=obj)



import contextlib
import hashlib
import logging
from functools import wraps
from typing import Callable
from typing import Iterable
from typing import Mapping
from typing import Optional

from django.core.cache import cache
from django.utils.encoding import force_bytes
from redis.exceptions import LockError


logger = logging.getLogger(__name__)

NO_CACHE = object()
DEFAULT_LOCK_TIMEOUT = 10
KeyMakerType = Callable[[Iterable, Mapping], str]


def redis_client(write=True):
    return cache.client.get_client(write=write)


def _make_key_id(*args, **kwargs):
    return ':'.join((
        *(str(value) for value in args),
        *(f'{key}={value}' for key, value in kwargs.items())
    ))


def _hash(key):
    return hashlib.md5(force_bytes(key)).hexdigest()  # noqa: S303


@contextlib.contextmanager
def use_lock(key, timeout=DEFAULT_LOCK_TIMEOUT, blocking_timeout=None):
    """
    If blocking_timeout is specified, will proceed after
    blocking_timeout even without the lock obtained.
    Check the return value and abort if needed.
    """

    if timeout is None:  # Prevent locks without timeout set - it is too dangerous if the process dies
        timeout = DEFAULT_LOCK_TIMEOUT

    lock = redis_client().lock(f'use_lock:{key}', timeout=timeout, blocking_timeout=blocking_timeout)
    acquired = lock.acquire(blocking=True)
    try:
        yield acquired
    finally:
        if not acquired:
            return

        # Don't want failed lock release to break the whole application
        try:
            lock.release()
        except LockError as exc:
            logger.exception(str(exc))


def with_lock(
    key,
    timeout,
    blocking_timeout=None,
    proceed_without_lock=False,
):
    """
    If blocking_timeout is specified, and the lock is not obtained after blocking_timeout,
    behavior will depend on the proceed_without_lock.
    """

    def factory(func):
        @wraps(func)
        def decorator(*args, **kwargs):
            with use_lock(key, timeout, blocking_timeout) as acquired:
                if not (acquired or proceed_without_lock):
                    return None
                return func(*args, **kwargs)

        return decorator

    return factory


def ensure_single(key, timeout):
    """Ensure function executes only single time simultaneously, other executions are aborted immediately"""
    return with_lock(key, timeout, blocking_timeout=0, proceed_without_lock=False)


def cache_memoize(
    timeout: Optional[int],
    fresh_after: Optional[int] = 0,
    prefix: Optional[int] = None,
    key_maker: Optional[KeyMakerType] = _make_key_id,
    calculation_time=DEFAULT_LOCK_TIMEOUT,
):
    if timeout is None and fresh_after:
        raise ValueError('fresh_after can only be specified when timeout is set')

    if fresh_after < 0:
        raise ValueError('fresh_after must be positive number')

    def factory(func):
        key_prefix = f'cache_memoize:{prefix or func.__name__}'

        def _make_cache_key(*args, **kwargs):  # noqa: WPS430
            # Allow to provide full key via key_prefix
            if key_maker is None:
                return key_prefix

            cache_key = _hash(key_maker(*args, **kwargs))
            return f'{key_prefix}:{cache_key}'

        @wraps(func)
        def decorator(*args, **kwargs):
            cache_key = _make_cache_key(*args, **kwargs)
            result = cache.get(cache_key, NO_CACHE)

            semi_fresh = False
            if result is not NO_CACHE:
                semi_fresh = fresh_after and (cache.ttl(cache_key) < fresh_after)
                if not semi_fresh:
                    # Still absolutely fresh
                    return result

            # Let one process to recalculate and others to return semi-fresh value (if exists),
            # else wait for calculation
            blocking_timeout = 0 if semi_fresh else None
            with use_lock(cache_key, timeout=calculation_time, blocking_timeout=blocking_timeout) as acquired:
                if not acquired:
                    # Only possible with semi_fresh = True (blocking_timeout = 0, no waiting for the lock)
                    return result

                if not semi_fresh:
                    # Check value wan't calculated while we were waiting for the lock
                    result = cache.get(cache_key, NO_CACHE)

                # This process was selected for value calculation
                if semi_fresh or result is NO_CACHE:
                    result = func(*args, **kwargs)
                    cache.set(cache_key, result, timeout + fresh_after)
            return result

        def invalidate(*args, **kwargs):  # noqa: WPS430
            cache_key = _make_cache_key(*args, **kwargs)
            return cache.delete(cache_key)

        decorator.invalidate = invalidate
        return decorator

    return factory



import unittest

from django.db import connection
from django.test.runner import DiscoverRunner


class FixedTestLoader(unittest.TestLoader):
    def _find_test_path(self, full_path, *args, **kwargs):
        # Don't look for tests inside models folder - this will cause invalid models import and RuntimeError
        if full_path.endswith('/models'):
            return None, False
        return super()._find_test_path(full_path, *args, **kwargs)


class FixedDiscoverRunner(DiscoverRunner):
    test_loader = FixedTestLoader()


def run_on_commit():
    while connection.run_on_commit:
        _, func = connection.run_on_commit.pop(0)
        func()



class classproperty:
    def __init__(self, method=None):
        self.fget = method

    def __get__(self, instance, cls=None):
        return self.fget(cls)


class Members:
    # noinspection PyPep8Naming
    @classproperty
    def ALL(self):
        return tuple(
            v for k, v in self.__dict__.items()
            if not k.startswith('__') and isinstance(v, str)
        )

    # noinspection PyPep8Naming
    @classproperty
    def CHOICES(self):
        return Choices(*self.ALL)


class Choices:
    def __init__(self, *choices):
        self._map = {}
        self._choices = ()
        for choice in choices:
            if isinstance(choice, (list, tuple)):
                db_value, human_name = choice
            else:
                db_value, human_name = (choice, choice)

            self._map[db_value] = human_name
            self._choices += ((db_value, human_name),)

    def __len__(self):
        return len(self._choices)

    def __iter__(self):
        return iter(self._choices)

    def __getitem__(self, key):
        return self._map[key]



import collections
from typing import Sized


# TODO: using . as a separator with dicts leads to ambiguity when keys contains dot. Use fields list instead
def get_attribute(instance, name):
    """
        Similar to Python's built in `getattr(instance, attr)`,
        but takes a list of nested attributes, instead of a single attribute.

        Also accepts either attribute lookup on objects or dictionary lookups.
    """

    attrs = name.split('.')
    for attr in attrs:
        if isinstance(instance, collections.Mapping):
            try:
                instance = instance[attr]
            except KeyError as exc:
                raise AttributeError(exc) from exc
        else:
            instance = getattr(instance, attr)
    return instance


def has_attribute(obj, name):
    """
        Like normal hasattr, but follows dotted paths
    """
    try:
        get_attribute(obj, name)
    except AttributeError:
        return False

    return True


def is_empty(val):
    """
        Check where value is logically `empty` - does not contain information.
        False and 0 are not considered empty, but empty collections are.
    """
    if val is None or isinstance(val, Sized) and len(val) == 0:  # Empty string is also Sized of len 0
        return True
    return False



from typing import Hashable, Iterable, List

from django_handy.attrs import get_attribute


def unique_ordered(sequence: Iterable[Hashable]) -> List:
    return list(dict.fromkeys(sequence))


def get_unique_objs(objs: List[object], unique_attrs: List[str]) -> List[object]:
    """
       Get list of unique objs from sequence,
        preserving order when the objs first occurred in original sequence
    """

    seen_obj_footprints = set()
    unique_objs = []
    for obj in objs:
        obj_footprint = tuple(get_attribute(obj, field) for field in unique_attrs)
        if obj_footprint in seen_obj_footprints:
            continue

        seen_obj_footprints.add(obj_footprint)
        unique_objs.append(obj)
    return unique_objs



from django.core.paginator import Paginator as DjangoPaginator
from django.utils.functional import cached_property
from rest_framework.pagination import LimitOffsetPagination
from rest_framework.pagination import PageNumberPagination
from rest_framework.response import Response

from django_handy.models import get_count


class DefaultPaginator(PageNumberPagination):
    page_size = 20
    page_size_query_param = 'page_size'
    max_page_size = 300

    def get_paginated_response(self, data):
        return Response({
            'count': self.page.paginator.count,
            'next': self.get_next_link(),
            'previous': self.get_previous_link(),
            'page': self.page.number,
            self.page_size_query_param: self.get_page_size(self.request),
            'results': data,
        })


class OptimizedDjangoPaginator(DjangoPaginator):
    @cached_property
    def count(self):
        return get_count(self.object_list)


class OptimizedDefaultPaginator(DefaultPaginator):
    django_paginator_class = OptimizedDjangoPaginator


class OptimizedLimitOffsetPaginator(LimitOffsetPagination):
    def get_count(self, queryset):
        return get_count(queryset)



from django import template
from django.conf import settings
from django.templatetags import static
from django.utils.safestring import mark_safe

from django_handy.url import simple_urljoin


register = template.Library()


@register.simple_tag
def mailto(address, text=None):
    return mark_safe(
        f'<a href="mailto:{address}">{text or address}</a>'
    )


if getattr(settings, 'HOST', None):
    class FullStaticNode(static.StaticNode):
        def url(self, context):
            relative_url = super().url(context)
            return simple_urljoin(settings.HOST, relative_url)


    @register.tag('fullstatic')
    def do_static(parser, token):
        return FullStaticNode.handle_token(parser, token)



from functools import wraps

from django.db import transaction


def call_on_commit(func):
    """
        Only call the decorated function at transaction commit.
        The return value will be ignored
    """

    @wraps(func)
    def handle(*args, **kwargs):
        transaction.on_commit(lambda: func(*args, **kwargs))

    return handle



try:
    from django.contrib.postgres.fields import ArrayField
except ImportError:
    ArrayField = None

import unicodedata

from django import forms
from django.core.validators import MinValueValidator
from django.db import models

from django_handy.unique import unique_ordered
from django_handy.validation import MinValueExclusionValidator


class LowerFieldDescriptor:
    def __init__(self, field):
        self.field = field

    def __get__(self, instance, owner=None):
        if instance is None:
            return None
        return instance.__dict__[self.field.attname]  # noqa: WPS609

    def __set__(self, instance, value):
        instance.__dict__[self.field.attname] = self.field.to_python(value)  # noqa: WPS609


class LowerCaseFieldMixin:
    def to_python(self, value):
        value = super().to_python(value)
        if value:
            value = unicodedata.normalize('NFKC', value)
            if hasattr(value, 'casefold'):  # noqa: WPS421
                value = value.casefold()
        return value

    def contribute_to_class(self, cls, name, **kwargs):  # noqa: WPS117
        super().contribute_to_class(cls, name, **kwargs)
        setattr(cls, self.attname, LowerFieldDescriptor(self))


class EmailLowerCaseField(LowerCaseFieldMixin, models.EmailField):
    pass


class DefaultDecimalField(models.DecimalField):
    def __init__(self, max_digits=15, decimal_places=2, **kwargs):
        super().__init__(max_digits=max_digits, decimal_places=decimal_places, **kwargs)


class PositiveDecimalField(DefaultDecimalField):
    default_validators = [MinValueValidator(0)]

    def __init__(self, no_zero=False, **kwargs):
        if no_zero:
            validators = kwargs.pop('validators', [])
            validators.append(MinValueExclusionValidator(0))
            kwargs['validators'] = validators
        super().__init__(**kwargs)


if ArrayField:
    class UniqueArrayField(ArrayField):
        """Ensures that no duplicates are saved to database"""

        def get_db_prep_value(self, value, connection, prepared=False):
            if isinstance(value, (list, tuple)):
                value = unique_ordered(value)
            return super().get_db_prep_value(value, connection, prepared)


    class ChoicesUniqueArrayField(UniqueArrayField):
        def formfield(self, **kwargs):
            defaults = {
                'form_class': forms.MultipleChoiceField,
                'choices': self.base_field.choices,
                'widget': forms.CheckboxSelectMultiple,
            }
            defaults.update(kwargs)
            # This super() call is intentional to jump over ArrayField.formfield
            return super(ArrayField, self).formfield(**defaults)



from rest_framework.mixins import UpdateModelMixin


class HiddenAttributesMeta(type):
    """Raise AttributeError when accessing hidden_attributes on class itself"""

    def __getattribute__(self, name):
        if name in super().__getattribute__('hidden_attributes'):
            raise AttributeError(name)
        return super().__getattribute__(name)


class PutModelMixin(UpdateModelMixin, metaclass=HiddenAttributesMeta):
    hidden_attributes = ['partial_update']



from rest_framework import status
from rest_framework.exceptions import APIException
from rest_framework.views import exception_handler


class ProcessingError(Exception):
    default_detail = 'An error occurred'

    def __init__(self, detail=None):
        if isinstance(detail, ProcessingError):
            detail = detail.detail
        self.detail = detail or self.default_detail

    def __repr__(self):
        return self.detail


class CommonException(APIException):
    status_code = status.HTTP_400_BAD_REQUEST
    default_detail = 'Something was wrong with provided data'

    def __init__(self, detail=None):
        self.detail = detail or self.default_detail


def custom_exception_handler(exc, context):
    if isinstance(exc, ProcessingError):
        exc = CommonException(detail=exc.detail)
    return exception_handler(exc, context)



import os
import re
import sys


try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


def get_version(*file_paths):
    """Retrieves the version from django_handy/__init__.py"""
    filename = os.path.join(os.path.dirname(__file__), *file_paths)
    version_file = open(filename).read()
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError('Unable to find version string.')


version = get_version("django_handy", "__init__.py")

if sys.argv[-1] == 'publish':
    try:
        import wheel

        print("Wheel version: ", wheel.__version__)
    except ImportError:
        print('Wheel library missing. Please run "pip install wheel"')
        sys.exit()
    os.system('python setup.py sdist upload')
    os.system('python setup.py bdist_wheel upload')
    sys.exit()

if sys.argv[-1] == 'tag':
    print("Tagging the version on git:")
    os.system("git tag -a %s -m 'version %s'" % (version, version))
    os.system("git push --tags")
    sys.exit()

setup(
    name='django-handy',
    version=version,
    license="MIT",
    description='Set of handy helpers for developing Django web applications.',
    long_description=open('README.rst').read(),
    author='Alexandr Tatarinov',
    author_email='tatarinov1997@gmail.com',
    url='https://github.com/alextatarinov/django-handy',
    packages=[
        'django_handy'
    ],
    install_requires=['Django>=2.2', 'djangorestframework>=3.9', 'django-manager-utils>=1.4', 'Pillow'],
    include_package_data=True,
    keywords='django-handy',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Framework :: Django',
        'Framework :: Django :: 2.2',
        'Framework :: Django :: 3.0',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    zip_safe=False,
    tests_require=['Django>=2.2', 'djangorestframework>=3.9', 'django-manager-utils>=1.4', 'psycopg2-binary'],
)
